@@ -2,7 +2,7 @@
  * sturmbahnfahrer
  * (c) 2006-2008 by Bram Stolk
  * bram at gmail.com
- * LICENSED ACCORDING TO THE GPL
+ * LICENSED ACCORDING TO THE GPLV3
  */
 
 #include "dynamicobject.h"
@@ -39,7 +39,6 @@ CarObject::CarObject
   ssgEntity *coilspringmodel,
   ssgEntity *leafspringmodel,
   ssgEntity *rearaxlemodel,
-  ssgEntity *licplatemodel,
   dWorldID world, 
   dSpaceID bigspace,
   sgVec3 initialpos
@@ -51,6 +50,7 @@ brake(0),
 steer(0),
 downtime(0),
 stationarytime(0),
+wheelspintime(0),
 action(false),
 reversegear(false),
 ebrake(false),
@@ -66,7 +66,6 @@ snapshot_available(false)
 
   branch->addKid(chassis_trf);
   chassis_trf->addKid(bodymodel);
-  if (licplatemodel) chassis_trf->addKid(licplatemodel);
   int i;
   for (i=0; i<4; i++)
   {
@@ -250,14 +249,6 @@ bool CarObject::IsStationary(void) const
   dReal magnitude = dSqrt(v[0]*v[0]+v[1]*v[1]+v[2]*v[2]);
   bool stationary = magnitude < 0.18;
 
-  // HACK: Turn Table has rotational velocity, which is hard to interpret.
-  // We simply assume we are stationary, so reverse gear is not blocked.
-  if (ground_object)
-  { 
-    if (ground_object->name == "turntablewheel")
-      stationary = true;
-  }
-
   return stationary;
 }
 
@@ -275,10 +266,22 @@ void CarObject::Simulate(float dt)
 
   // Are we in reverse gear?
   if (IsStationary())
+  {
     stationarytime += dt;
+    if (!ebrake && (accelerator > 0.7f || brake > 0.7))
+      wheelspintime += dt;
+    else
+      wheelspintime = 0;
+  }
   else
+  {
     stationarytime = 0;
-  if (stationarytime>0.15 && brake>0)
+    wheelspintime = 0;
+  }
+  // HACK: Turn Table has rotational velocity, which is hard to interpret.
+  // We simply assume we are stationary, so reverse gear is not blocked.
+  bool hack = (ground_object && ground_object->name == "turntablewheel");
+  if ((stationarytime>0.15||hack) && brake>0)
     reversegear=true;
   if (brake==0)
     reversegear=false;
@@ -375,6 +378,15 @@ void CarObject::Recover(float dt)
     dBodyAddForce(wheel_bodies[3], -f[0],-f[1],-f[2]);
     sign=-sign;
   }
+
+  if (wheelspintime > 1.0f)
+  {
+    wheelspintime = 0.0f;
+    float x = ((rand()&255)-128) / 64.0f / dt;
+    float y = ((rand()&255)-128) / 64.0f / dt;
+    float z = 16.0 / dt;
+    dBodyAddForce(chassis_body, x,y,z);
+  }
 }
 
 
@@ -391,7 +403,7 @@ void CarObject::UpdateSpindles(float travel_l, float travel_r)
   sgSetVec3(m[3], HWHEELBASE, SPINDLEY, -travel_l );
   spindle_trfs[0]->setTransform(m);
 
-  sgMakeRotMat4(m, 180.0 * angle_r / M_PI, axis);
+  sgMakeRotMat4(m, -180.0 * angle_r / M_PI, axis);
   sgSetVec3(m[3], HWHEELBASE, -SPINDLEY, -travel_r );
   spindle_trfs[1]->setTransform(m);
 }
