@@ -2,7 +2,7 @@
  * stormbaancoureur
  * (c) 2006,2007,2008 by Bram Stolk
  * bram at gmail.com
- * LICENSED ACCORDING TO THE GPL
+ * LICENSED ACCORDING TO THE GPLV3
  */
 
 #include <assert.h>
@@ -20,6 +20,7 @@
 #include <plib/ssg.h>
 #include <plib/ul.h>
 
+#include "ogl.h"
 #include "staticworldobject.h"
 #include "carobject.h"
 #include "trackingcam.h"
@@ -36,7 +37,6 @@
 #include "postscore.h"
 #include "sturmbahn.h"
 #include "starsky.h"
-#include "texture.h"
 
 static float aspectratio=1.0;
 static int winw=704, winh=300;
@@ -47,7 +47,8 @@ static StereoContext *stereocontext=0;
 static ssgContext    *lightviewcontext=0;
 static float fps=60.0;
 static float gametime=0.0;
-static char *username="anonymou";
+static float spawntime=0.0;
+static const char *username="anonymou";
 static float dt_hist[10]={0.016,0.016,0.016,0.016,0.016,0.016,0.016,0.016,0.016,0.016};
 static std::string dirprefix;
 
@@ -76,295 +77,15 @@ static bool capture=false;
 static ModelMap *modelmap=0;
 static StarSky  *starsky=0;
 
-// Variables used for hardware accelerated shadow mapping
 static bool do_shadows;
-static GLuint frame_buffer_object;
-static GLuint light_view_depth_texture;
-static Texture *spotlight_texture=0;
-static const int shadowmapsize = 1024;
-static const float spot_fov = 3.250f;
-static GLuint fragment_shader, vertex_shader;
-static GLuint program;
-static GLint uniform_loc_tex0, uniform_loc_tex1, uniform_loc_tex2;
-
 
 static void stop_game(void);
 static void start_game(const std::string &gametype);
+static void restart_game(void);
 static bool respawn(void);
 
 
 
-static void OglErrorCheck(const std::string &context)
-{
-  GLenum error = glGetError();
-  while (error != GL_NO_ERROR)
-  {
-    fprintf(stderr, "plode OpenGL error (%s): %s\n", context.c_str(),gluErrorString(error));
-    error = glGetError();
-  }
-}
-
-
-/*
- * This helper function checks wether we have all req'd extensions for shadow mapping
- */
-static bool OglCanDoShadowing(void)
-{
-  if (getenv("PLODE_NO_SHADOWS")) return false;
-  const char *vers = (const char*) glGetString(GL_VERSION);
-  const char *rend = (const char*) glGetString(GL_RENDERER);
-  fprintf(stderr, "OpenGL is version %s\n", vers);
-  fprintf(stderr, "OpenGL renderer %s\n", rend);
-  const char *exts = (const char*) glGetString(GL_EXTENSIONS);
-  //fprintf(stderr, "OpenGL has extensions %s\n", exts);
-  if (!strstr(exts, "GL_ARB_multitexture")) return false;
-  if (!strstr(exts, "GL_ARB_depth_texture")) return false;
-  if (!strstr(exts, "GL_ARB_shadow")) return false;
-  if (!strstr(exts, "GL_EXT_framebuffer_object")) return false;
-  if (!strstr(exts, "GL_ARB_vertex_shader")) return false;
-  if (!strstr(exts, "GL_ARB_fragment_shader")) return false;
-  if (!strstr(exts, "GL_ARB_shader_objects")) return false;
-  return true;
-}
-
-
-static void OglPrepareShaders(void)
-{
-  PFNGLCREATESHADERPROC  glCreateShader  = (PFNGLCREATESHADERPROC)  glXGetProcAddressARB((const GLubyte *) "glCreateShader");
-  PFNGLSHADERSOURCEPROC  glShaderSource  = (PFNGLSHADERSOURCEPROC)  glXGetProcAddressARB((const GLubyte *) "glShaderSource");
-  PFNGLCOMPILESHADERPROC glCompileShader = (PFNGLCOMPILESHADERPROC) glXGetProcAddressARB((const GLubyte *) "glCompileShader");
-  PFNGLGETSHADERIVPROC   glGetShaderiv   = (PFNGLGETSHADERIVPROC)   glXGetProcAddressARB((const GLubyte *) "glGetShaderiv");
-  PFNGLCREATEPROGRAMPROC glCreateProgram = (PFNGLCREATEPROGRAMPROC) glXGetProcAddressARB((const GLubyte *) "glCreateProgram");
-  PFNGLATTACHSHADERPROC  glAttachShader  = (PFNGLATTACHSHADERPROC)  glXGetProcAddressARB((const GLubyte *) "glAttachShader");
-  PFNGLLINKPROGRAMPROC   glLinkProgram   = (PFNGLLINKPROGRAMPROC)   glXGetProcAddressARB((const GLubyte *) "glLinkProgram");
-  PFNGLGETPROGRAMIVPROC  glGetProgramiv  = (PFNGLGETPROGRAMIVPROC)  glXGetProcAddressARB((const GLubyte *) "glGetProgramiv");
-  PFNGLGETSHADERINFOLOGPROC glGetShaderInfoLog = (PFNGLGETSHADERINFOLOGPROC) glXGetProcAddressARB((const GLubyte *) "glGetShaderInfoLog");
-  PFNGLGETPROGRAMINFOLOGPROC glGetProgramInfoLog = (PFNGLGETPROGRAMINFOLOGPROC) glXGetProcAddressARB((const GLubyte *) "glGetProgramInfoLog");
-  PFNGLGETUNIFORMLOCATIONPROC glGetUniformLocation = (PFNGLGETUNIFORMLOCATIONPROC) glXGetProcAddressARB((const GLubyte *) "glGetUniformLocation");
-
-  assert(glCreateShader);
-  assert(glShaderSource);
-  assert(glCompileShader);
-  assert(glGetShaderiv);
-  assert(glCreateProgram);
-  assert(glLinkProgram);
-  assert(glAttachShader);
-  assert(glGetProgramiv);
-  assert(glGetShaderInfoLog);
-  assert(glGetUniformLocation);
-
-  static GLchar fragment_program_text[8192];
-  static GLchar vertex_program_text[8192];
-  const char *fp = fragment_program_text;
-  const char *vp = vertex_program_text;
-  char log_text[4096];
-  int sz;
-  FILE *f;
-  std::string fname;
-  GLint status;
-
-  fname = dirprefix + "/shaders/bramlight.fp";
-  f = fopen(fname.c_str(),"rb"); assert(f); sz=fread(fragment_program_text, 1, 8192, f); fclose(f);
-  sz = strlen(fragment_program_text);
-  fragment_shader = glCreateShader(GL_FRAGMENT_SHADER);
-  glShaderSource(fragment_shader, 1, &fp, 0);
-  glCompileShader(fragment_shader);
-  glGetShaderiv(fragment_shader, GL_COMPILE_STATUS, &status);
-  if (status != GL_TRUE)
-  {
-    glGetShaderInfoLog(fragment_shader, sizeof(log_text), 0, log_text);
-    fprintf(stderr,"%s\n",log_text);
-    exit(1);
-  }
-
-  fname = dirprefix + "/shaders/bramlight.vp";
-  f = fopen(fname.c_str(),"rb"); assert(f); sz=fread(vertex_program_text, 1, 8192, f); fclose(f);
-  sz = strlen(vertex_program_text);
-  vertex_shader = glCreateShader(GL_VERTEX_SHADER);
-  glShaderSource(vertex_shader, 1, &vp, 0);
-  glCompileShader(vertex_shader);
-  glGetShaderiv(vertex_shader, GL_COMPILE_STATUS, &status);
-  if (status != GL_TRUE)
-  {
-    glGetShaderInfoLog(vertex_shader, sizeof(log_text), 0, log_text);
-    fprintf(stderr,"%s\n",log_text);
-    exit(1);
-  }
-
-  program = glCreateProgram();
-  glAttachShader(program, vertex_shader);
-  glAttachShader(program, fragment_shader);
-  glLinkProgram(program);
-  glGetProgramiv(program, GL_LINK_STATUS, &status);
-  if (status != GL_TRUE)
-  {
-    glGetProgramInfoLog(program, sizeof(log_text), 0, log_text);
-    fprintf(stderr,"%s\n",log_text);
-    exit(1);
-  }
-
-  // Find the locations of the uniforms that we are using.
-  uniform_loc_tex0 = glGetUniformLocation(program, "texture0");
-  uniform_loc_tex1 = glGetUniformLocation(program, "texture1");
-  uniform_loc_tex2 = glGetUniformLocation(program, "texture2");
-  assert(uniform_loc_tex0 != -1);
-  assert(uniform_loc_tex1 != -1);
-  assert(uniform_loc_tex2 != -1);
-
-  OglErrorCheck("prepare fragment shader");
-}
-
-
-/*
- * This piece of code sets up a frame buffer object, associated with a texture.
- * We can render a depth-only image as seen from the light, into this texture.
- */
-static void OglInitShadowing(void)
-{
-  // create texture for shadow map
-  glGenTextures(1, &light_view_depth_texture);
-  glBindTexture(GL_TEXTURE_2D, light_view_depth_texture);
-  glEnable(GL_DEPTH_TEST);
-  GLint bits;
-  glGetIntegerv(GL_DEPTH_BITS, &bits);
-  glTexImage2D
-  (
-    GL_TEXTURE_2D,
-    0,
-    (bits==16) ? GL_DEPTH_COMPONENT16_ARB : GL_DEPTH_COMPONENT24_ARB,
-    shadowmapsize, shadowmapsize, 0,
-    GL_DEPTH_COMPONENT, GL_UNSIGNED_INT, 0
-  );
-  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
-  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
-  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
-  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
-  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_COMPARE_MODE_ARB, GL_COMPARE_R_TO_TEXTURE_ARB);
-  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_COMPARE_FUNC_ARB, GL_LEQUAL);
-  OglErrorCheck("creating shadow texture");
-
-  // create texture for spotlight image
-  spotlight_texture = new Texture(dirprefix+"/images/spot.tga");
-  spotlight_texture->InitForRendering();
-  OglErrorCheck("creating spotlight texture");
-
-  // Get the addresses of the extended functionality.
-  PFNGLGENFRAMEBUFFERSEXTPROC glGenFramebuffers = (PFNGLGENFRAMEBUFFERSEXTPROC) glXGetProcAddressARB((const GLubyte *) "glGenFramebuffersEXT");
-  PFNGLBINDFRAMEBUFFEREXTPROC glBindFramebuffer = (PFNGLBINDFRAMEBUFFEREXTPROC) glXGetProcAddressARB((const GLubyte *) "glBindFramebufferEXT");
-  PFNGLFRAMEBUFFERTEXTURE2DEXTPROC glFramebufferTexture2D = (PFNGLFRAMEBUFFERTEXTURE2DEXTPROC) glXGetProcAddressARB((const GLubyte *) "glFramebufferTexture2DEXT");
-  PFNGLCHECKFRAMEBUFFERSTATUSEXTPROC glCheckFramebufferStatus = (PFNGLCHECKFRAMEBUFFERSTATUSEXTPROC) glXGetProcAddressARB((const GLubyte *) "glCheckFramebufferStatusEXT");
-
-  assert(glGenFramebuffers);
-  assert(glBindFramebuffer);
-  assert(glFramebufferTexture2D);
-  assert(glCheckFramebufferStatus);
-
-  // create fbo
-  glGenFramebuffers(1, &frame_buffer_object);
-  glBindFramebuffer(GL_FRAMEBUFFER_EXT, frame_buffer_object);
-  // Attach texture to fbo
-  glFramebufferTexture2D
-  (
-    GL_FRAMEBUFFER_EXT,
-    GL_DEPTH_ATTACHMENT_EXT,
-    GL_TEXTURE_2D,
-    light_view_depth_texture,
-   0
-  );
-
-  // We use depth render only
-  glDrawBuffer(GL_NONE);
-  glReadBuffer(GL_NONE);
-
-  GLenum status = glCheckFramebufferStatus(GL_FRAMEBUFFER_EXT);
-  assert(status != GL_FRAMEBUFFER_UNSUPPORTED_EXT);
-  //fprintf(stderr,"status %x\n", status);
-  assert(status == GL_FRAMEBUFFER_COMPLETE_EXT);
-
-  // disable the frame buffer object
-  glBindFramebuffer(GL_FRAMEBUFFER_EXT, 0);
-  OglErrorCheck("creating frame buffer object");
-
-  OglPrepareShaders();
-}
-
-
-static void set_linear_texgen(GLenum plane, GLenum mode)
-{
-  // eye linear
-  GLfloat xx[4]={1,0,0,0};
-  GLfloat yy[4]={0,1,0,0};
-  GLfloat zz[4]={0,0,1,0};
-  GLfloat ww[4]={0,0,0,1};
-  glTexGenfv(GL_S, plane, xx);
-  glTexGenfv(GL_T, plane, yy);
-  glTexGenfv(GL_R, plane, zz);
-  glTexGenfv(GL_Q, plane, ww);
-  glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, mode);
-  glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, mode);
-  glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, mode);
-  glTexGeni(GL_Q, GL_TEXTURE_GEN_MODE, mode);
-  glEnable(GL_TEXTURE_GEN_S);
-  glEnable(GL_TEXTURE_GEN_T);
-  glEnable(GL_TEXTURE_GEN_R);
-  glEnable(GL_TEXTURE_GEN_Q);
-}
-
-
-static void render_shadow_map(void)
-{
-  // Prepare shadow rendering
-  int port[4];
-  glGetIntegerv(GL_VIEWPORT, port);
-  PFNGLBINDFRAMEBUFFEREXTPROC glBindFramebuffer = (PFNGLBINDFRAMEBUFFEREXTPROC) glXGetProcAddressARB((const GLubyte *) "glBindFramebufferEXT");
-  glBindFramebuffer(GL_FRAMEBUFFER_EXT, frame_buffer_object);
-  glViewport(0,0, shadowmapsize,shadowmapsize);
-  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); // depth only?
-//  glPolygonOffset(2.5, 10);
-  glPolygonOffset(1.1, 4.0);
-  glEnable(GL_POLYGON_OFFSET_FILL);
-  OglErrorCheck("prepare shadow rendering");
-
-  // render scene from light view
-
-  glActiveTextureARB(GL_TEXTURE1_ARB);
-
-  glPushMatrix();
-  set_linear_texgen(GL_EYE_PLANE, GL_EYE_LINEAR);
-  glPopMatrix();
-
-  glMatrixMode(GL_TEXTURE);
-  glLoadIdentity();
-  glTranslatef(0.5f, 0.5f, 0.5f);
-  glScalef(0.5f, 0.5f, 0.5f);
-  gluPerspective(spot_fov, 1, 0.1, 150.0);
-  glMatrixMode(GL_MODELVIEW);
-
-  spotlight_texture->Activate();
-  glEnable(GL_TEXTURE_2D);
-  glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
-
-  // Render the scene
-  glActiveTextureARB(GL_TEXTURE0_ARB);
-  set_linear_texgen(GL_OBJECT_PLANE, GL_OBJECT_LINEAR);
-  ssgCullAndDraw(scene);
-  OglErrorCheck("ssgCullAndDraw");
-
-  glActiveTextureARB(GL_TEXTURE1_ARB);
-  glBindTexture(GL_TEXTURE_2D, 0);
-  glDisable(GL_TEXTURE_2D);
-  glActiveTextureARB(GL_TEXTURE0_ARB);
-  glBindTexture(GL_TEXTURE_2D, 0);
-  glDisable(GL_TEXTURE_2D);
-
-  // reset viewport and polygon offset to original values.
-  glDisable(GL_POLYGON_OFFSET_FILL);
-  glBindFramebuffer(GL_FRAMEBUFFER_EXT, 0);
-  glViewport(port[0], port[1], port[2], port[3]);
-  OglErrorCheck("render_shadow_map");
-}
-
-
 static void near_callback(void *data, dGeomID o1, dGeomID o2)
 {
   assert(o1);
@@ -445,22 +166,26 @@ static void near_callback(void *data, dGeomID o1, dGeomID o2)
     for (int i=0; i<n; i++)
     {
       contact[i].surface.mu = 50.0;
-      contact[i].surface.soft_erp = 0.985;
-      contact[i].surface.soft_cfm = 0.020;
+      //contact[i].surface.soft_erp = 0.985;
+      //contact[i].surface.soft_cfm = 0.020;
+      //contact[i].surface.mode = dContactSoftERP | dContactSoftCFM | dContactApprox1;
+      contact[i].surface.mode = dContactApprox1;
 
-      contact[i].surface.mode = dContactSoftERP | dContactSoftCFM | dContactApprox1;
       if (is_tyre) 
       {
         contact[i].fdir1[0]=wheel_dir[0];
         contact[i].fdir1[1]=wheel_dir[1];
         contact[i].fdir1[2]=wheel_dir[2];
-        contact[i].surface.slip1 = 0.04f * wheel_vel; // slip perpendicular to roll direction
-        contact[i].surface.slip2 = 0.01f * wheel_vel; // slip in roll direction
-        contact[i].surface.mode = contact[i].surface.mode | dContactFDir1 | dContactSlip1 | dContactSlip2 | dContactSoftERP | dContactSoftCFM;
-        contact[i].surface.mu  = 400.0;
-        contact[i].surface.mu2 = 400.0;
+        contact[i].surface.slip1 = 0.0250f * wheel_vel; // slip perpendicular to roll direction
+        contact[i].surface.slip2 = 0.0005f * wheel_vel; // slip in roll direction
+        contact[i].surface.mode = contact[i].surface.mode | dContactFDir1 | dContactSlip1 | dContactSlip2;
+#if 0
         contact[i].surface.soft_erp = 0.95;
         contact[i].surface.soft_cfm = 0.05;
+        contact[i].surface.mode = contact[i].surface.mode | dContactSoftERP | dContactSoftCMF;
+#endif
+        contact[i].surface.mu  = 340.0;
+        contact[i].surface.mu2 = 340.0;
       }
 
       if ((wo1->name == "ferriswheelcart" || wo2->name == "ferriswheelcart") && is_tyre)
@@ -585,10 +310,11 @@ static void idle(void)
   car->SetEBrake(ebr);
 
   gametime += elapsed;
+  spawntime += elapsed;
   if (sengine)
     engine_noise(dt);
 
-  float timestep=0.008333; // Use 8.333ms timesteps
+  float timestep=0.005555; // Use 5.555ms timesteps
   static float remaining_sim_time=0;
   remaining_sim_time += dt;
 
@@ -624,16 +350,7 @@ static void idle(void)
   car->GetPos(carpos);
   if (carpos[2] < -4)
   {
-    const char *gametype="";
-    if (sturmbahnpractice) gametype="practice";
-    if (sturmbahnrally) gametype="rally";
-    if (sturmbahnfull) gametype="full";
-    bool respawned = respawn();
-    if (!respawned)
-    {
-      stop_game();
-      start_game(gametype);
-    }
+    restart_game();
     return;
   }
 
@@ -690,16 +407,12 @@ static void redraw_single_channel(void)
 {
   if (scene && !intro && (!gui || !gui->hasmenu))
   {
-    static PFNGLUSEPROGRAMPROC glUseProgram = (PFNGLUSEPROGRAMPROC) glXGetProcAddressARB((const GLubyte *) "glUseProgram");
-    static PFNGLUNIFORM1IPROC  glUniform1i  = (PFNGLUNIFORM1IPROC)  glXGetProcAddressARB((const GLubyte *) "glUniform1i");
-
     if (do_shadows)
     {
-      assert(glUseProgram);
       // 1st pass from light view
 
       lightviewcontext->makeCurrent();
-      render_shadow_map();
+      OglRenderShadowMap(scene);
 
       sgMat4 mat_cam, mat_spt;
       if (monocontext)
@@ -714,47 +427,7 @@ static void redraw_single_channel(void)
       }
       lightviewcontext->getModelviewMatrix(mat_spt);
 
-      // 2nd pass: with shadow texture
-      glUseProgram(program);
-      glActiveTextureARB(GL_TEXTURE1_ARB);
-      glUniform1i(uniform_loc_tex1, 1);
-      glPushMatrix();
-      glMultMatrixf((GLfloat*)mat_cam);
-      set_linear_texgen(GL_EYE_PLANE, GL_EYE_LINEAR);
-      glPopMatrix();
-      glMatrixMode(GL_TEXTURE);
-      glLoadIdentity();
-      glTranslatef(0.5f,0.5f,0.5f);
-      glScalef(0.5f,0.5f,0.5f);
-      gluPerspective(spot_fov, 1, 1.0, 150.0);
-      glMultMatrixf((GLfloat*)mat_spt);
-      glMatrixMode(GL_MODELVIEW);
-      spotlight_texture->Activate();
-      glEnable(GL_TEXTURE_2D);
-      glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
-
-      glActiveTextureARB(GL_TEXTURE2_ARB);
-      glUniform1i(uniform_loc_tex2, 2);
-      glPushMatrix();
-      glMultMatrixf((GLfloat*)mat_cam);
-      set_linear_texgen(GL_EYE_PLANE, GL_EYE_LINEAR);
-      glPopMatrix();
-      glMatrixMode(GL_TEXTURE);
-      glLoadIdentity();
-      glTranslatef(0.5f, 0.5f, 0.5f); // r coord scale
-      glScalef(0.5f, 0.5f, 0.5f); // r coord bias
-      gluPerspective(spot_fov, 1, 1.0f, 150.0f);
-      glMultMatrixf((GLfloat*)mat_spt);
-      OglErrorCheck("setup texture matrix");
-
-      glMatrixMode(GL_MODELVIEW);
-      glBindTexture(GL_TEXTURE_2D, light_view_depth_texture);
-      glEnable(GL_TEXTURE_2D);
-      glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
- 
-      glActiveTextureARB(GL_TEXTURE0_ARB);
-      glUniform1i(uniform_loc_tex0, 0);
-      set_linear_texgen(GL_OBJECT_PLANE, GL_OBJECT_LINEAR);
+      OglSetupSecondPass((float*)mat_cam, (float*)mat_spt);
     }
 
     ssgCullAndDraw(scene);
@@ -907,12 +580,12 @@ static void special(int k, int, int)
 {
   if (k == GLUT_KEY_UP)
   {
-    if (gui) gui->ChangeChoice(-1);
+    if (gui && gui->hasmenu) gui->ChangeChoice(-1);
     if (controller_key) controller_key->FeedAction("ACCEL", true);
   }
   if (k == GLUT_KEY_DOWN)
   {
-    if (gui) gui->ChangeChoice(1);
+    if (gui && gui->hasmenu) gui->ChangeChoice(1);
     if (controller_key) controller_key->FeedAction("BRAKE", true);
   }
   if (k == GLUT_KEY_LEFT)
@@ -984,6 +657,7 @@ static void motion(int x, int y)
 static void start_game(const std::string &gametype)
 {
   gametime = 0.0;
+  spawntime = 0.0;
 
   // Create ODE world
   dInitODE();
@@ -991,11 +665,15 @@ static void start_game(const std::string &gametype)
   bigspace = dHashSpaceCreate(0);
   staticspace = dSimpleSpaceCreate(bigspace);
   contactgroup = dJointGroupCreate (0);
+#if 0
+  dWorldSetERP(world, 0.95f);
+  dWorldSetCFM(world, 0.001f);
+#endif
   dWorldSetGravity(world,0,0,-9.8);
   dWorldSetAutoDisableFlag(world, true);
   dWorldSetAutoDisableLinearThreshold(world, 0.04);
   dWorldSetAutoDisableAngularThreshold(world, 0.04);
-  dWorldSetQuickStepNumIterations(world, 64);
+  dWorldSetQuickStepNumIterations(world, 16);
 
   // Create PLIB world
 
@@ -1021,11 +699,6 @@ static void start_game(const std::string &gametype)
   if (sturmbahnrally)
     sgSetVec3(carpos, -28, 0, 1.5);
 
-  ssgEntity *licplatemodel = 0;
-#if 0
-  if (!modelmap->LowDetail())
-    licplatemodel = modelmap->Get("licplate.ac");
-#endif
   car = new CarObject
   (
     modelmap->Get("frame.3ds"),
@@ -1035,7 +708,6 @@ static void start_game(const std::string &gametype)
     modelmap->Get("coilspring.3ds"),
     modelmap->Get("leafspring.3ds"),
     modelmap->Get("rearaxle.3ds"),
-    licplatemodel,
     world,
     bigspace,
     carpos
@@ -1084,6 +756,7 @@ static void stop_game(void)
 
 static bool respawn(void)
 {
+  spawntime=0.0f;
   bool restored = car->SnapShot(true);
   if (!restored)
     return false;
@@ -1091,6 +764,21 @@ static bool respawn(void)
 }
 
 
+static void restart_game(void)
+{
+  const char *gametype="";
+  if (sturmbahnpractice) gametype="practice";
+  if (sturmbahnrally) gametype="rally";
+  if (sturmbahnfull) gametype="full";
+  bool respawned = respawn();
+  if (!respawned)
+  {
+    stop_game();
+    start_game(gametype);
+  }
+}
+
+
 static void keyboard(unsigned char k, int, int)
 {
   if (k==27)
@@ -1109,9 +797,10 @@ static void keyboard(unsigned char k, int, int)
       }
       exit(0);
     }
-    stop_game();
-    gametime=0.0;
-    if (gui) gui->hasmenu=true;
+    if (spawntime>1.0f)
+      restart_game();
+    else
+      if (gui) gui->hasmenu=true;
   }
 
   if (gui && gui->hasmenu)
@@ -1165,7 +854,7 @@ int main(int argc, char *argv[])
   {
     username = unam;
     if (strlen(unam)>8)
-      username[8]=0;
+      unam[8]=0;
   }
   glutInit(&argc, argv);
   int flags = GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH | GLUT_MULTISAMPLE;
@@ -1184,7 +873,20 @@ int main(int argc, char *argv[])
   glutMouseFunc(mouse);
   glutMotionFunc(motion);
   glutIdleFunc(idle);
+
+#if 1
   glutFullScreen();
+#else
+  if (1)
+  {
+    int width = glutGameModeGet( GLUT_GAME_MODE_WIDTH );
+    int height = glutGameModeGet( GLUT_GAME_MODE_HEIGHT );
+    int pixelDepth = glutGameModeGet( GLUT_GAME_MODE_PIXEL_DEPTH );
+    int refreshRate = glutGameModeGet( GLUT_GAME_MODE_REFRESH_RATE );
+    fprintf(stderr,"%dx%d-%d@%d\n", width, height, pixelDepth, refreshRate);
+    //glutEnterGameMode();
+  }
+#endif
 
   do_shadows = OglCanDoShadowing();
   fprintf
@@ -1193,13 +895,14 @@ int main(int argc, char *argv[])
     "This platform %s all required GL extensions to do hardware accelerated shadowing.\n",
     (do_shadows) ? "supports" : "does not support"
   );
-  if (do_shadows)
-  {
-    OglInitShadowing();
-  }
 
   ssgInit();
 
+  if (do_shadows)
+  {
+    OglInitShadowing(dirprefix);
+  }
+
   glEnable(GL_DEPTH_TEST);
   glEnable(GL_CULL_FACE);
 
