@@ -21,7 +21,7 @@
 
 #include "postscore.h"
 
-static const char *hostname = "leaderboard.stolk.org";
+static const char *hostname = "www.stolk.org";
 static const int portnr=7460;
 
 static char msg[1472];
@@ -38,8 +38,10 @@ void postscore_put(const char *username, float tim)
 
   struct hostent *he = gethostbyname(hostname);
   if (!he)
+  {
     perror("gethostbyname");
-  assert(he);
+    return;
+  }
   struct in_addr ip_addr = *(struct in_addr *)(he->h_addr);
   char *ipnr = inet_ntoa(ip_addr);
 
